@@ -1,5 +1,5 @@
 /* gifview.c - gifview's main loop.
-   Copyright (C) 1997-2000 Eddie Kohler, eddietwo@lcs.mit.edu
+   Copyright (C) 1997-2001 Eddie Kohler, eddietwo@lcs.mit.edu
    This file is part of gifview, in the gifsicle package.
 
    Gifview is free software. It is distributed under the GNU Public License,
@@ -475,6 +475,7 @@ delete_viewer(Gt_Viewer *viewer)
   else viewers = viewer->next;
   
   Gif_DeleteStream(viewer->gfs);
+  Gif_DeleteArray(viewer->unoptimized_pixmaps);
   Gif_DeleteArray(viewer->im);
   Gif_DeleteXContext(viewer->gfx);
   Gif_Delete(viewer);
@@ -1082,12 +1083,17 @@ loop(void)
   while (viewers) {
     
     /* Check for any animations */
-    while (animations && xwTIMEGEQ(now, animations->timer)) {
+    /* 13.Feb.2001 - Use the 'pending' counter to avoid a tight loop
+       if all the frames in an animation have delay 0. Reported by
+       Franc,ois Petitjean. */
+    pending = 0;
+    while (animations && xwTIMEGEQ(now, animations->timer) && pending < 25) {
       v = animations;
       animations = v->anim_next;
       v->scheduled = 0;
       view_frame(v, v->im_pos + 1);
       xwGETTIME(now);
+      pending++;
     }
     
     pending = XPending(display);
@@ -1214,7 +1220,7 @@ main(int argc, char **argv)
       
      case VERSION_OPT:
       printf("gifview (LCDF Gifsicle) %s\n", VERSION);
-      printf("Copyright (C) 1997-2000 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2001 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
