@@ -1,20 +1,23 @@
 /* gifx.c - Functions to turn GIFs in memory into X Pixmaps.
-   Copyright (C) 1997-8 Eddie Kohler, eddietwo@lcs.mit.edu
+   Copyright (C) 1997-9 Eddie Kohler, eddietwo@lcs.mit.edu
    This file is part of the GIF library.
 
-   The GIF library is free software*. It is distributed under the GNU Public
-   License, version 2 or later; you can copy, distribute, or alter it at will,
-   as long as this notice is kept intact and this source code is made
+   The GIF library is free software*. It is distributed under the GNU General
+   Public License, version 2 or later; you can copy, distribute, or alter it
+   at will, as long as this notice is kept intact and this source code is made
    available. There is no warranty, express or implied.
 
    *The LZW compression method used by GIFs is patented. Unisys, the patent
    holder, allows the compression algorithm to be used without a license in
    software distributed at no cost to the user. */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include "gifx.h"
-#include "gif.h"
 #include <X11/Xutil.h>
 #include <assert.h>
+#include <string.h>
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -28,7 +31,7 @@ struct Gif_XColormap {
   
   int allocated;
   int claimed;
-  u_int16_t npixels;
+  uint16_t npixels;
   unsigned long *pixels;
   
   Gif_XColormap *next;
@@ -42,8 +45,8 @@ static void
 load_closest(Gif_XContext *gfx)
 {
   XColor *color;
-  u_int16_t ncolor;
-  u_int16_t ncolormap;
+  uint16_t ncolor;
+  uint16_t ncolormap;
   int i;
   
   if (gfx->closest) return;
@@ -80,7 +83,7 @@ allocate_closest(Gif_XContext *gfx, Gif_Color *c)
 {
   Gif_Color *closer;
   Gif_Color *got = 0;
-  u_int32_t distance = 0x4000000;
+  uint32_t distance = 0x4000000;
   int i;
   
   load_closest(gfx);
@@ -89,7 +92,7 @@ allocate_closest(Gif_XContext *gfx, Gif_Color *c)
     int redd = c->red - closer->red;
     int greend = c->green - closer->green;
     int blued = c->blue - closer->blue;
-    u_int32_t d = redd * redd + greend * greend + blued * blued;
+    uint32_t d = redd * redd + greend * greend + blued * blued;
     if (d < distance) {
       distance = d;
       got = closer;
@@ -120,7 +123,7 @@ static void
 allocate_colors(Gif_XColormap *gfxc)
 {
   Gif_XContext *gfx = gfxc->x_context;
-  u_int16_t size = gfxc->colormap->ncol;
+  uint16_t size = gfxc->colormap->ncol;
   Gif_Color *c = gfxc->colormap->col;
   unsigned long *pixels = gfxc->pixels;
   XColor xcol;
@@ -159,7 +162,7 @@ create_x_colormap_extension(Gif_XContext *gfx, Gif_Colormap *gfcm)
   unsigned long *pixels;
   if (!gfcm) return 0;
   gfxc = Gif_New(Gif_XColormap);
-  pixels = gfxc ? Gif_NewArray(unsigned long, gfcm->ncol) : 0;
+  pixels = gfxc ? Gif_NewArray(unsigned long, 256) : 0;
   if (pixels) {
     gfxc->x_context = gfx;
     gfxc->colormap = gfcm;
@@ -260,12 +263,11 @@ Gif_XClaimStreamColors(Gif_XContext *gfx, Gif_Stream *gfs, int *np_store)
 
 #define BYTESIZE 8
 
-Pixmap
-Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
-		      Gif_Colormap *gfcm, Gif_Image *gfi,
-		      int left, int top, int width, int height)
+static int
+put_sub_image_colormap(Gif_XContext *gfx, Gif_Image *gfi, Gif_Colormap *gfcm,
+		       int left, int top, int width, int height,
+		       Pixmap pixmap, int pixmap_x, int pixmap_y)
 {
-  Pixmap pixmap = None;
   XImage *ximage;
   byte *xdata;
   
@@ -274,12 +276,15 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
   
   unsigned long saved_transparent = 0;
   int release_uncompressed = 0;
-  u_int16_t nct;
+  uint16_t nct;
   unsigned long *pixels;
   
   /* Find the correct image and colormap */
-  if (!gfi && gfs->nimages) gfi = gfs->images[0];
-  if (!gfi) return None;
+  if (!gfi) return 0;
+  if (!gfx->image_gc)
+    gfx->image_gc = XCreateGC(gfx->display, pixmap, 0, 0);
+  if (!gfx->image_gc)
+    return 0;
 
   /* Make sure the image is uncompressed */
   if (!gfi->img && !gfi->image_data && gfi->compressed) {
@@ -291,13 +296,13 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
   if (width <= 0 || height <= 0 || left < 0 || top < 0
       || left+width <= 0 || top+height <= 0
       || left+width > gfi->width || top+height > gfi->height)
-    return None;
+    return 0;
   
   /* Allocate colors from the colormap; make sure the transparent color
    * has the given pixel value */
   if (gfcm) {
     Gif_XColormap *gfxc = find_x_colormap_extension(gfx, gfcm, 1);
-    if (!gfxc) return None;
+    if (!gfxc) return 0;
     allocate_colors(gfxc);
     pixels = gfxc->pixels;
     nct = gfxc->npixels;
@@ -306,7 +311,7 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
     pixels = crap_pixels;
     nct = 256;
   }
-  if (gfi->transparent > -1 && gfi->transparent < nct) {
+  if (gfi->transparent >= 0 && gfi->transparent < 256) {
     saved_transparent = pixels[ gfi->transparent ];
     pixels[ gfi->transparent ] = gfx->transparent_pixel;
   }
@@ -349,11 +354,11 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
   } else {
     /* Other bits-per-pixel */
     int bits_per_pixel = ximage->bits_per_pixel;
-    u_int32_t bits_per_pixel_mask = (1UL << bits_per_pixel) - 1;
+    uint32_t bits_per_pixel_mask = (1UL << bits_per_pixel) - 1;
     
     for (j = 0; j < height; j++) {
       int imshift = 0;
-      u_int32_t impixel = 0;
+      uint32_t impixel = 0;
       byte *line = gfi->img[top + j] + left;
       byte *writer = xdata + bytes_per_line * j;
       
@@ -379,18 +384,12 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
   }
   
   /* Restore saved transparent pixel value */
-  if (gfi->transparent > -1 && gfi->transparent < nct)
+  if (gfi->transparent >= 0 && gfi->transparent < 256)
     pixels[ gfi->transparent ] = saved_transparent;
 
-  /* Create the pixmap */
-  pixmap =
-    XCreatePixmap(gfx->display, gfx->drawable, width, height, gfx->depth);
-  if (!gfx->image_gc)
-    gfx->image_gc = XCreateGC(gfx->display, pixmap, 0, 0);
-  
-  if (pixmap && gfx->image_gc)
-    XPutImage(gfx->display, pixmap, gfx->image_gc, ximage, 0, 0, 0, 0,
-	      width, height);
+  /* Put it onto the pixmap */
+  XPutImage(gfx->display, pixmap, gfx->image_gc, ximage, 0, 0,
+	    pixmap_x, pixmap_y, width, height);
   
   Gif_DeleteArray(xdata);
   ximage->data = 0; /* avoid freeing it again in XDestroyImage */  
@@ -399,10 +398,26 @@ Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Stream *gfs,
   if (release_uncompressed)
     Gif_ReleaseUncompressedImage(gfi);
   
-  return pixmap;
+  return 1;
 }
 
 
+Pixmap
+Gif_XSubImageColormap(Gif_XContext *gfx, Gif_Image *gfi, Gif_Colormap *gfcm,
+		      int left, int top, int width, int height)
+{
+  Pixmap pixmap =
+    XCreatePixmap(gfx->display, gfx->drawable, width, height, gfx->depth);
+  if (pixmap) {
+    if (put_sub_image_colormap(gfx, gfi, gfcm, left, top, width, height,
+			       pixmap, 0, 0))
+      return pixmap;
+    else
+      XFreePixmap(gfx->display, pixmap);
+  }
+  return None;
+}
+
 Pixmap
 Gif_XImage(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi)
 {
@@ -411,22 +426,20 @@ Gif_XImage(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi)
   if (!gfi) return None;
   gfcm = gfi->local;
   if (!gfcm) gfcm = gfs->global;
-  return Gif_XSubImageColormap(gfx, gfs, gfcm, gfi,
+  return Gif_XSubImageColormap(gfx, gfi, gfcm,
 			       0, 0, gfi->width, gfi->height);
 }
 
-
 Pixmap
 Gif_XImageColormap(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Colormap *gfcm,
 		   Gif_Image *gfi)
 {
   if (!gfi && gfs->nimages) gfi = gfs->images[0];
   if (!gfi) return None;
-  return Gif_XSubImageColormap(gfx, gfs, gfcm, gfi,
+  return Gif_XSubImageColormap(gfx, gfi, gfcm,
 			       0, 0, gfi->width, gfi->height);
 }
 
-
 Pixmap
 Gif_XSubImage(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi,
 	      int left, int top, int width, int height)
@@ -436,13 +449,13 @@ Gif_XSubImage(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi,
   if (!gfi) return None;
   gfcm = gfi->local;
   if (!gfcm) gfcm = gfs->global;
-  return Gif_XSubImageColormap(gfx, gfs, gfcm, gfi,
+  return Gif_XSubImageColormap(gfx, gfi, gfcm,
 			       left, top, width, height);
 }
 
 
 Pixmap
-Gif_XSubMask(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi,
+Gif_XSubMask(Gif_XContext *gfx, Gif_Image *gfi,
 	     int left, int top, int width, int height)
 {
   Pixmap pixmap = None;
@@ -455,7 +468,6 @@ Gif_XSubMask(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi,
   int release_uncompressed = 0;
   
   /* Find the correct image */
-  if (!gfi && gfs->nimages) gfi = gfs->images[0];
   if (!gfi) return None;
   
   /* Check subimage dimensions */
@@ -487,7 +499,7 @@ Gif_XSubMask(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi,
   /* The main loop */
   for (j = 0; j < height; j++) {
     int imshift = 0;
-    u_int32_t impixel = 0;
+    uint32_t impixel = 0;
     byte *line = gfi->img[top + j] + left;
     byte *writer = xdata + bytes_per_line * j;
     
@@ -532,7 +544,107 @@ Gif_XMask(Gif_XContext *gfx, Gif_Stream *gfs, Gif_Image *gfi)
 {
   if (!gfi && gfs->nimages) gfi = gfs->images[0];
   if (!gfi) return None;
-  return Gif_XSubMask(gfx, gfs, gfi, 0, 0, gfi->width, gfi->height);
+  return Gif_XSubMask(gfx, gfi, 0, 0, gfi->width, gfi->height);
+}
+
+
+Pixmap
+Gif_XNextImage(Gif_XContext *gfx, Pixmap last_last, Pixmap last,
+	       Gif_Stream *gfs, int n)
+{
+  Pixmap pixmap = None, image = None, mask = None;
+  Gif_Image *gfi = gfs->images[n];
+  Gif_Image *last_gfi = (n > 0 ? gfs->images[n-1] : 0);
+  Gif_Colormap *gfcm = (gfi->local ? gfi->local : gfs->global);
+  unsigned long bg_pixel;
+  unsigned long old_transparent = gfx->transparent_pixel;
+
+  if (gfs->screen_width == 0 || gfs->screen_height == 0)
+    return None;
+  
+  if (gfi->width == gfs->screen_width && gfi->height == gfs->screen_height
+      && gfi->left == 0 && gfi->top == 0 && gfi->transparent < 0)
+    return Gif_XImage(gfx, gfs, gfi);
+  
+  /* create the pixmap */
+  pixmap = XCreatePixmap(gfx->display, gfx->drawable,
+			 gfs->screen_width, gfs->screen_height, gfx->depth);
+  if (!pixmap) goto error_exit;
+  
+  /* find background color if necessary */
+  if (last == None
+      || (last_gfi && last_gfi->disposal == GIF_DISPOSAL_BACKGROUND)) {
+    /* find bg_pixel */
+    if (gfs->global && gfs->background < gfs->global->ncol
+	&& gfs->images[0]->transparent < 0) {
+      Gif_XColormap *gfxc = find_x_colormap_extension(gfx, gfcm, 1);
+      if (!gfxc)
+	return None;
+      allocate_colors(gfxc);
+      bg_pixel = gfxc->pixels[gfs->background];
+    } else
+      bg_pixel = old_transparent;
+    /* install it as the foreground color on gfx->image_gc */
+    if (!gfx->image_gc)
+      gfx->image_gc = XCreateGC(gfx->display, pixmap, 0, 0);
+    if (!gfx->image_gc) goto error_exit;
+    XSetForeground(gfx->display, gfx->image_gc, bg_pixel);
+    gfx->transparent_pixel = bg_pixel;
+  }
+  
+  /* if there is no `last' then we need special handling */
+  if (last == None
+      || (last_gfi && last_gfi->width == gfs->screen_width
+	  && last_gfi->height == gfs->screen_height
+	  && last_gfi->left == 0 && last_gfi->top == 0
+	  && last_gfi->disposal == GIF_DISPOSAL_BACKGROUND)) {
+    XFillRectangle(gfx->display, pixmap, gfx->image_gc, 0, 0,
+		   gfs->screen_width, gfs->screen_height);
+    if (!put_sub_image_colormap(gfx, gfi, gfcm, 0, 0, gfi->width, gfi->height,
+				pixmap, gfi->left, gfi->top))
+      goto error_exit;
+    return pixmap;
+  }
+  
+  /* use the disposal to create the intermediate image */
+  XCopyArea(gfx->display, last, pixmap, gfx->image_gc,
+	    0, 0, gfs->screen_width, gfs->screen_height, 0, 0);
+  if (last_last != None && last_gfi->disposal == GIF_DISPOSAL_PREVIOUS)
+    XCopyArea(gfx->display, last_last, pixmap, gfx->image_gc,
+	      last_gfi->left, last_gfi->top, last_gfi->width, last_gfi->height,
+	      last_gfi->left, last_gfi->top);
+  else if (last_gfi->disposal == GIF_DISPOSAL_BACKGROUND)
+    XFillRectangle(gfx->display, pixmap, gfx->image_gc,
+		   last_gfi->left, last_gfi->top,
+		   last_gfi->width, last_gfi->height);
+
+  /* apply image */
+  if (gfi->transparent < 0) {
+    if (!put_sub_image_colormap(gfx, gfi, gfcm, 0, 0, gfi->width, gfi->height,
+				pixmap, gfi->left, gfi->top))
+      goto error_exit;
+  } else {
+    image = Gif_XImage(gfx, gfs, gfi);
+    mask = Gif_XMask(gfx, gfs, gfi);
+    if (image == None || mask == None) goto error_exit;
+    XSetClipMask(gfx->display, gfx->image_gc, mask);
+    XSetClipOrigin(gfx->display, gfx->image_gc, gfi->left, gfi->top);
+    XCopyArea(gfx->display, image, pixmap, gfx->image_gc,
+	      0, 0, gfi->width, gfi->height, gfi->left, gfi->top);
+    XSetClipMask(gfx->display, gfx->image_gc, None);
+    XFreePixmap(gfx->display, image);
+    XFreePixmap(gfx->display, mask);
+  }
+  
+  gfx->transparent_pixel = old_transparent;
+  return pixmap;
+  
+ error_exit:
+  if (pixmap) XFreePixmap(gfx->display, pixmap);
+  if (image) XFreePixmap(gfx->display, image);
+  if (mask) XFreePixmap(gfx->display, mask);
+  gfx->transparent_pixel = old_transparent;
+  return None;
 }
 
 
