@@ -136,6 +136,7 @@ General options: Also --no-OPTION for info and verbose.\n\
       --version                 Print version number and exit.\n\
   -o, --output FILE             Write output to FILE.\n\
   -w, --no-warnings             Don't report warnings.\n\
+      --conserve-memory         Conserve memory at the expense of speed.\n\
 \n", program_name);
   printf("\
 Frame selections:               #num, #num1-num2, #num1-, #name\n\
@@ -244,7 +245,7 @@ verbose_endline(void)
 
 
 static void
-safe_puts(const char *s, u_int32_t len, FILE *f)
+safe_puts(const char *s, uint32_t len, FILE *f)
 {
   const char *last_safe = s;
   for (; len > 0; len--, s++)
@@ -307,8 +308,8 @@ static void
 extension_info(FILE *where, Gif_Stream *gfs, Gif_Extension *gfex, int count)
 {
   byte *data = gfex->data;
-  u_int32_t pos = 0;
-  u_int32_t len = gfex->length;
+  uint32_t pos = 0;
+  uint32_t len = gfex->length;
   
   fprintf(where, "  extension %d: ", count);
   if (gfex->kind == 255) {
@@ -328,8 +329,8 @@ extension_info(FILE *where, Gif_Stream *gfs, Gif_Extension *gfex, int count)
   
   /* Now, hexl the data. */
   while (len > 0) {
-    u_int32_t row = 16;
-    u_int32_t i;
+    uint32_t row = 16;
+    uint32_t i;
     if (row > len) row = len;
     fprintf(where, "    %08x: ", pos);
     
@@ -1162,7 +1163,7 @@ fix_total_crop(Gif_Stream *dest, Gif_Image *srci, int merger_index)
 
 
 static void
-handle_screen(Gif_Stream *dest, u_int16_t width, u_int16_t height)
+handle_screen(Gif_Stream *dest, uint16_t width, uint16_t height)
 {
   /* Set the screen width & height, if the current input width and height are
      larger */
@@ -1178,8 +1179,8 @@ handle_flip_and_screen(Gif_Stream *dest, Gif_Image *desti,
 {
   Gif_Stream *gfs = fr->stream;
   
-  u_int16_t screen_width = gfs->screen_width;
-  u_int16_t screen_height = gfs->screen_height;
+  uint16_t screen_width = gfs->screen_width;
+  uint16_t screen_height = gfs->screen_height;
   
   if (fr->flip_horizontal)
     flip_image(desti, screen_width, screen_height, 0);
@@ -1274,11 +1275,11 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
     int s;
     for (i = s = 0; i < nmerger; i++)
       s += ((merger[i]->image->width * merger[i]->image->height) / 1024) + 1;
-    *huge_stream = (s > 20 * 1024); /* 20 MB */
-    if (*huge_stream && compress_immediately < 0)
-      warning("working on a huge stream -- this may take a while, or fail");
-    else if (*huge_stream)
+    *huge_stream = (s > 200 * 1024); /* 200 MB */
+    if (*huge_stream && !compress_immediately) {
+      warning("huge GIF, conserving memory (processing may take a while)");
       compress_immediately = 1;
+    }
   }
   
   /* merge stream-specific info and clear colormaps */
@@ -1419,9 +1420,15 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
       desti->disposal = fr->disposal;
 
     /* compress immediately if possible to save on memory */
-    if (compress_immediately && desti->img) {
-      Gif_FullCompressImage(dest, desti, gif_write_flags);
-      Gif_ReleaseUncompressedImage(desti);
+    if (desti->img) {
+      if (compress_immediately > 0) {
+	Gif_FullCompressImage(dest, desti, gif_write_flags);
+	Gif_ReleaseUncompressedImage(desti);
+      } else if (desti->compressed)
+	Gif_ReleaseCompressedImage(desti);
+    } else if (compress_immediately <= 0) {
+      Gif_UncompressImage(desti);
+      Gif_ReleaseCompressedImage(desti);
     }
     
    merge_frame_done:
