@@ -1,3 +1,4 @@
+/* -*- mode: c; c-basic-offset: 2 -*- */
 /* ungifwrt.c - Functions to write unGIFs -- GIFs with run-length compression,
    not LZW compression. Idea due to Hutchinson Avenue Software Corporation
    <http://www.hasc.com>.
@@ -35,11 +36,13 @@ typedef struct Gif_Writer {
   
   FILE *f;
   byte *v;
-  u_int32_t pos;
-  u_int32_t cap;
+  uint32_t pos;
+  uint32_t cap;
   int flags;
+  int global_size;
+  int local_size;
   void (*byte_putter)(byte, struct Gif_Writer *);
-  void (*block_putter)(byte *, u_int16_t, struct Gif_Writer *);
+  void (*block_putter)(byte *, uint16_t, struct Gif_Writer *);
   
 } Gif_Writer;
 
@@ -48,7 +51,7 @@ typedef struct Gif_Writer {
 #define gifputblock(b, l, grr)	((*grr->block_putter)(b, l, grr))
 
 static inline void
-gifputunsigned(u_int16_t uns, Gif_Writer *grr)
+gifputunsigned(uint16_t uns, Gif_Writer *grr)
 {
   gifputbyte(uns & 0xFF, grr);
   gifputbyte(uns >> 8, grr);
@@ -62,7 +65,7 @@ file_byte_putter(byte b, Gif_Writer *grr)
 }
 
 static void
-file_block_putter(byte *block, u_int16_t size, Gif_Writer *grr)
+file_block_putter(byte *block, uint16_t size, Gif_Writer *grr)
 {
   fwrite(block, size, 1, grr->f);
 }
@@ -82,7 +85,7 @@ memory_byte_putter(byte b, Gif_Writer *grr)
 }
 
 static void
-memory_block_putter(byte *data, u_int16_t len, Gif_Writer *grr)
+memory_block_putter(byte *data, uint16_t len, Gif_Writer *grr)
 {
   if (grr->pos + len >= grr->cap) {
     grr->cap *= 2;
@@ -95,19 +98,155 @@ memory_block_putter(byte *data, u_int16_t len, Gif_Writer *grr)
 }
 
 
-#ifdef GIF_NO_COMPRESSION
+#ifndef GIF_NO_COMPRESSION
+
+/* Write GIFs compressed with run-length encoding, an idea from code by
+   Hutchinson Avenue Software Corporation <http://www.hasc.com> found in
+   Thomas Boutell's gd library <http://www.boutell.com>. */
 
 static void
-real_write_image_data(byte **img, u_int16_t width, u_int16_t height,
+write_compressed_data(byte **img, uint16_t width, uint16_t height,
 		      byte min_code_bits, Gif_Context *gfc, Gif_Writer *grr)
 {
   byte buffer[WRITE_BUFFER_SIZE];
   byte *buf;
   
-  u_int16_t xleft;
+  uint16_t xleft;
   byte *imageline;
   
-  u_int32_t leftover;
+  uint32_t leftover;
+  byte bits_left_over;
+  
+  Gif_Code next_code;
+  Gif_Code output_code;
+  Gif_Code clear_code;
+  Gif_Code eoi_code;
+#define CUR_BUMP_CODE (1 << cur_code_bits)
+  byte suffix;
+  
+  Gif_Code *rle_next = gfc->rle_next;
+  
+  byte cur_code_bits;
+  
+  /* Here we go! */
+  gifputbyte(min_code_bits, grr);
+  clear_code = 1 << min_code_bits;
+  eoi_code = clear_code + 1;
+  
+  cur_code_bits = min_code_bits + 1;
+  /* next_code set by first runthrough of output clear_code */
+  GIF_DEBUG(("clear(%d) eoi(%d) bits(%d)",clear_code,eoi_code,cur_code_bits));
+  
+  output_code = clear_code;
+  /* Because output_code is clear_code, we'll initialize next_code, et al.
+     below. */
+  
+  bits_left_over = 0;
+  leftover = 0;
+  buf = buffer;
+  xleft = width;
+  imageline = img[0];
+  
+  while (1) {
+    
+    /*****
+     * Output `output_code' to the data stream. */
+
+    leftover |= output_code << bits_left_over;
+    bits_left_over += cur_code_bits;
+    while (bits_left_over >= 8) {
+      *buf++ = leftover & 0xFF;
+      leftover = (leftover >> 8) & 0x00FFFFFF;
+      bits_left_over -= 8;
+      if (buf == buffer + WRITE_BUFFER_SIZE) {
+	gifputbyte(WRITE_BUFFER_SIZE, grr);
+	gifputblock(buffer, WRITE_BUFFER_SIZE, grr);
+	buf = buffer;
+      }
+    }
+    
+    if (output_code == clear_code) {
+      /* Clear data and prepare gfc */
+      Gif_Code c;
+      
+      cur_code_bits = min_code_bits + 1;
+      next_code = eoi_code + 1;
+      
+      for (c = 0; c < clear_code; c++)
+	rle_next[c] = clear_code;
+      
+    } else if (next_code > CUR_BUMP_CODE) {
+      /* bump up compression size */
+      if (cur_code_bits == GIF_MAX_CODE_BITS) {
+	output_code = clear_code;
+	continue;
+      } else
+	cur_code_bits++;
+      
+    } else if (output_code == eoi_code)
+      break;
+    
+    
+    /*****
+     * Find the next code to output. */
+    
+    /* If height is 0 -- no more pixels to write -- output eoi_code. */
+    if (height == 0)
+      output_code = eoi_code;
+    else {
+      output_code = suffix = *imageline;
+      if (output_code >= clear_code)
+	/* should not happen unless GIF_WRITE_CAREFUL_MIN_CODE_BITS */
+	output_code = suffix = 0;
+      goto next_pixel;
+      
+      while (height != 0 && *imageline == suffix
+	     && rle_next[output_code] != clear_code) {
+	output_code = rle_next[output_code];
+       next_pixel:
+	imageline++;
+	xleft--;
+	if (xleft == 0) {
+	  xleft = width;
+	  height--;
+	  img++;
+	  imageline = img[0];
+	}
+      }
+      
+      if (height != 0 && *imageline == suffix) {
+	rle_next[output_code] = next_code;
+	rle_next[next_code] = clear_code;
+      }
+      next_code++;
+    }
+  }
+  
+  if (bits_left_over > 0)
+    *buf++ = leftover;
+  
+  if (buf != buffer) {
+    GIF_DEBUG(("imageblock(%d)", buf - buffer));
+    gifputbyte(buf - buffer, grr);
+    gifputblock(buffer, buf - buffer, grr);
+  }
+  
+  gifputbyte(0, grr);
+}
+
+#else /* GIF_NO_COMPRESSION */
+
+static void
+write_compressed_data(byte **img, uint16_t width, uint16_t height,
+		      byte min_code_bits, Gif_Context *gfc, Gif_Writer *grr)
+{
+  byte buffer[WRITE_BUFFER_SIZE];
+  byte *buf;
+  
+  uint16_t xleft;
+  byte *imageline;
+  
+  uint32_t leftover;
   byte bits_left_over;
   
   Gif_Code next_code;
@@ -209,144 +348,6 @@ real_write_image_data(byte **img, u_int16_t width, u_int16_t height,
   gifputbyte(0, grr);
 }
 
-#else /* GIF_NO_COMPRESSION */
-
-/* Write GIFs compressed with run-length encoding, an idea from code by
-   Hutchinson Avenue Software Corporation <http://www.hasc.com> found in
-   Thomas Boutell's gd library <http://www.boutell.com>. */
-
-static void
-real_write_image_data(byte **img, u_int16_t width, u_int16_t height,
-		      byte min_code_bits, Gif_Context *gfc, Gif_Writer *grr)
-{
-  byte buffer[WRITE_BUFFER_SIZE];
-  byte *buf;
-  
-  u_int16_t xleft;
-  byte *imageline;
-  
-  u_int32_t leftover;
-  byte bits_left_over;
-  
-  Gif_Code next_code;
-  Gif_Code output_code;
-  Gif_Code clear_code;
-  Gif_Code eoi_code;
-  Gif_Code bump_code;
-  byte suffix;
-  
-  Gif_Code *rle_next = gfc->rle_next;
-  
-  byte cur_code_bits;
-  
-  /* Here we go! */
-  gifputbyte(min_code_bits, grr);
-  clear_code = 1 << min_code_bits;
-  eoi_code = clear_code + 1;
-  
-  cur_code_bits = min_code_bits + 1;
-  /* next_code set by first runthrough of output clear_code */
-  GIF_DEBUG(("clear(%d) eoi(%d) bits(%d)",clear_code,eoi_code,cur_code_bits));
-  
-  output_code = clear_code;
-  /* Because output_code is clear_code, we'll initialize next_code, et al.
-     below. */
-  
-  bits_left_over = 0;
-  leftover = 0;
-  buf = buffer;
-  xleft = width;
-  imageline = img[0];
-  
-  while (1) {
-    
-    /*****
-     * Output `output_code' to the data stream. */
-
-    leftover |= output_code << bits_left_over;
-    bits_left_over += cur_code_bits;
-    while (bits_left_over >= 8) {
-      *buf++ = leftover & 0xFF;
-      leftover = (leftover >> 8) & 0x00FFFFFF;
-      bits_left_over -= 8;
-      if (buf == buffer + WRITE_BUFFER_SIZE) {
-	gifputbyte(WRITE_BUFFER_SIZE, grr);
-	gifputblock(buffer, WRITE_BUFFER_SIZE, grr);
-	buf = buffer;
-      }
-    }
-    
-    if (output_code == clear_code) {
-      /* Clear data and prepare gfc */
-      Gif_Code c;
-      
-      cur_code_bits = min_code_bits + 1;
-      next_code = eoi_code + 1;
-      bump_code = clear_code << 1;
-      
-      for (c = 0; c < clear_code; c++)
-	rle_next[c] = clear_code;
-      
-    } else if (next_code > bump_code) {
-      
-      /* bump up compression size */
-      if (cur_code_bits == GIF_MAX_CODE_BITS) {
-	output_code = clear_code;
-	continue;
-      } else {
-	cur_code_bits++;
-	bump_code <<= 1;
-      }
-      
-    } else if (output_code == eoi_code)
-      break;
-    
-    
-    /*****
-     * Find the next code to output. */
-    
-    /* If height is 0 -- no more pixels to write -- output eoi_code. */
-    if (height == 0)
-      output_code = eoi_code;
-    else {
-      output_code = suffix = *imageline;
-      if (output_code >= clear_code) output_code = 0;
-      goto next_pixel;
-      
-      while (height != 0 && *imageline == suffix
-	     && rle_next[output_code] != clear_code) {
-	output_code = rle_next[output_code];
-       next_pixel:
-	imageline++;
-	xleft--;
-	if (xleft == 0) {
-	  xleft = width;
-	  height--;
-	  img++;
-	  imageline = img[0];
-	}
-      }
-      
-      if (height != 0 && *imageline == suffix) {
-	rle_next[output_code] = next_code;
-	rle_next[next_code] = clear_code;
-      }
-      next_code++;
-    }
-  }
-  
-  if (bits_left_over > 0)
-    *buf++ = leftover;
-  
-  if (buf != buffer) {
-    GIF_DEBUG(("imageblock(%d)", buf - buffer));
-    gifputbyte(buf - buffer, grr);
-    gifputblock(buffer, buf - buffer, grr);
-  }
-  
-  gifputbyte(0, grr);
-}
-
 #endif /* GIF_NO_COMPRESSION */
 
 
@@ -357,10 +358,10 @@ calculate_min_code_bits(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 
   if (grr->flags & GIF_WRITE_CAREFUL_MIN_CODE_SIZE) {
     /* calculate m_c_b based on colormap */
-    if (gfi->local && gfi->local->ncol > 0)
-      colors_used = gfi->local->ncol;
-    else if (gfs->global && gfs->global->ncol > 0)
-      colors_used = gfs->global->ncol;
+    if (grr->local_size > 0)
+      colors_used = grr->local_size;
+    else if (grr->global_size > 0)
+      colors_used = grr->global_size;
     
   } else if (gfi->compressed) {
     /* take m_c_b from compressed image */
@@ -405,10 +406,10 @@ write_image_data(Gif_Image *gfi, byte min_code_bits,
 		 Gif_Context *gfc, Gif_Writer *grr)
 {
   byte **img = gfi->img;
-  u_int16_t width = gfi->width, height = gfi->height;
+  uint16_t width = gfi->width, height = gfi->height;
     
   if (gfi->interlace) {
-    u_int16_t y;
+    uint16_t y;
     byte **nimg = Gif_NewArray(byte *, height + 1);
     if (!nimg) return 0;
     
@@ -416,16 +417,18 @@ write_image_data(Gif_Image *gfi, byte min_code_bits,
       nimg[y] = img[Gif_InterlaceLine(y, height)];
     nimg[height] = 0;
     
-    real_write_image_data(nimg, width, height, min_code_bits, gfc, grr);
+    write_compressed_data(nimg, width, height, min_code_bits, gfc, grr);
     
     Gif_DeleteArray(nimg);
   } else
-    real_write_image_data(img, width, height, min_code_bits, gfc, grr);
+    write_compressed_data(img, width, height, min_code_bits, gfc, grr);
   
   return 1;
 }
 
 
+static int get_color_table_size(Gif_Stream *, Gif_Image *, Gif_Writer *);
+
 int
 Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi, int flags)
 {
@@ -438,8 +441,11 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi, int flags)
     (*gfi->free_compressed)((void *)gfi->compressed);
     gfi->compressed = 0;
   }
-  
-  gfc.rle_next = Gif_NewArray(Gif_Code, GIF_MAX_CODE);
+
+  /* 27.Jul.2001: Must allocate GIF_MAX_CODE + 1 because we assign to
+     rle_next[GIF_MAX_CODE]! Thanks, Jeff Brown <jabrown@ipn.caida.org>, for
+     supplying the buggy files. */
+  gfc.rle_next = Gif_NewArray(Gif_Code, GIF_MAX_CODE + 1);
   
   grr.v = Gif_NewArray(byte, 1024);
   grr.pos = 0;
@@ -447,6 +453,8 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi, int flags)
   grr.byte_putter = memory_byte_putter;
   grr.block_putter = memory_block_putter;
   grr.flags = flags;
+  grr.global_size = get_color_table_size(gfs, 0, &grr);
+  grr.local_size = get_color_table_size(gfs, gfi, &grr);
   
   if (!grr.v || !gfc.rle_next)
     goto done;
@@ -467,19 +475,46 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi, int flags)
 }
 
 
-static void
-write_color_table(Gif_Color *c, int ncol, Gif_Writer *grr)
+static int
+get_color_table_size(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 {
-  int totalcol, i;
+  Gif_Colormap *gfcm = (gfi ? gfi->local : gfs->global);
+  int ncol, totalcol, i;
+
+  if (!gfcm || gfcm->ncol <= 0)
+    return 0;
+
   /* Make sure ncol is reasonable */
-  if (ncol <= 0) ncol = 0;
-  if (ncol > 256) ncol = 256;
-  
+  ncol = gfcm->ncol;
+
+  /* Possibly bump up 'ncol' based on 'transparent' values, if
+     careful_min_code_bits */
+  if (grr->flags & GIF_WRITE_CAREFUL_MIN_CODE_SIZE) {
+    if (gfi && gfi->transparent >= ncol)
+      ncol = gfi->transparent + 1;
+    else if (!gfi)
+      for (i = 0; i < gfs->nimages; i++)
+	if (gfs->images[i]->transparent >= ncol)
+	  ncol = gfs->images[i]->transparent + 1;
+  }
+
   /* Make sure the colormap is a power of two entries! */
   /* GIF format doesn't allow a colormap with only 1 entry. */
-  for (totalcol = 2; totalcol < ncol; totalcol *= 2) ;
+  if (ncol > 256)
+    ncol = 256;
+  for (totalcol = 2; totalcol < ncol; totalcol *= 2)
+    /* nada */;
   
-  for (i = 0; i < ncol; i++, c++) {
+  return totalcol;
+}
+
+static void
+write_color_table(Gif_Colormap *gfcm, int totalcol, Gif_Writer *grr)
+{
+  Gif_Color *c = gfcm->col;
+  int i, ncol = gfcm->ncol;
+  
+  for (i = 0; i < ncol && i < totalcol; i++, c++) {
     gifputbyte(c->red, grr);
     gifputbyte(c->green, grr);
     gifputbyte(c->blue, grr);
@@ -498,6 +533,7 @@ static int
 write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
 {
   byte min_code_bits, packed = 0;
+  grr->local_size = get_color_table_size(gfs, gfi, grr);
   
   gifputbyte(',', grr);
   gifputunsigned(gfi->left, grr);
@@ -505,21 +541,21 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
   gifputunsigned(gfi->width, grr);
   gifputunsigned(gfi->height, grr);
   
-  if (gfi->local && gfi->local->ncol > 0) {
+  if (grr->local_size > 0) {
     int size = 2;
     packed |= 0x80;
-    while (size < gfi->local->ncol && size < 256)
+    while (size < grr->local_size)
       size *= 2, packed++;
   }
   
   if (gfi->interlace) packed |= 0x40;
   gifputbyte(packed, grr);
   
-  if (gfi->local && gfi->local->ncol > 0)
-    write_color_table(gfi->local->col, gfi->local->ncol, grr);
+  if (grr->local_size > 0)
+    write_color_table(gfi->local, grr->local_size, grr);
 
   /* calculate min_code_bits here (because calculation may involve
-     recompression, if GIF_WRITE_CAREFUL_MIN_CODE_BITS is true) */
+     recompression, if GIF_WRITE_CAREFUL_MIN_CODE_SIZE is true) */
   min_code_bits = calculate_min_code_bits(gfs, gfi, grr);
   
   /* use existing compressed data if it exists. This will tend to whip
@@ -527,9 +563,9 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Context *gfc, Gif_Writer *grr)
      but modify the uncompressed data anyway. That sucks. */
   if (gfi->compressed) {
     byte *compressed = gfi->compressed;
-    u_int32_t compressed_len = gfi->compressed_len;
+    uint32_t compressed_len = gfi->compressed_len;
     while (compressed_len > 0) {
-      u_int16_t amt = (compressed_len > 0x7000 ? 0x7000 : compressed_len);
+      uint16_t amt = (compressed_len > 0x7000 ? 0x7000 : compressed_len);
       gifputblock(compressed, amt, grr);
       compressed += amt;
       compressed_len -= amt;
@@ -546,15 +582,16 @@ static void
 write_logical_screen_descriptor(Gif_Stream *gfs, Gif_Writer *grr)
 {
   byte packed = 0x70;		/* high resolution colors */
+  grr->global_size = get_color_table_size(gfs, 0, grr);
 
   Gif_CalculateScreenSize(gfs, 0);
   gifputunsigned(gfs->screen_width, grr);
   gifputunsigned(gfs->screen_height, grr);
   
-  if (gfs->global) {
-    u_int16_t size = 2;
+  if (grr->global_size > 0) {
+    uint16_t size = 2;
     packed |= 0x80;
-    while (size < gfs->global->ncol && size < 256)
+    while (size < grr->global_size)
       size *= 2, packed++;
   }
   
@@ -562,8 +599,8 @@ write_logical_screen_descriptor(Gif_Stream *gfs, Gif_Writer *grr)
   gifputbyte(gfs->background, grr);
   gifputbyte(0, grr);		/* no aspect ratio information */
   
-  if (gfs->global)
-    write_color_table(gfs->global->col, gfs->global->ncol, grr);
+  if (grr->global_size > 0)
+    write_color_table(gfs->global, grr->global_size, grr);
 }
 
 
@@ -627,7 +664,7 @@ write_comment_extensions(Gif_Comment *gfcom, Gif_Writer *grr)
 
 
 static void
-write_netscape_loop_extension(u_int16_t value, Gif_Writer *grr)
+write_netscape_loop_extension(uint16_t value, Gif_Writer *grr)
 {
   gifputblock((byte *)"!\xFF\x0BNETSCAPE2.0\x03\x01", 16, grr);
   gifputunsigned(value, grr);
@@ -638,7 +675,7 @@ write_netscape_loop_extension(u_int16_t value, Gif_Writer *grr)
 static void
 write_generic_extension(Gif_Extension *gfex, Gif_Writer *grr)
 {
-  u_int32_t pos = 0;
+  uint32_t pos = 0;
   if (gfex->kind < 0) return;	/* ignore our private extensions */
   
   gifputbyte('!', grr);
@@ -656,7 +693,7 @@ write_generic_extension(Gif_Extension *gfex, Gif_Writer *grr)
     pos += 255;
   }
   if (pos < gfex->length) {
-    u_int32_t len = gfex->length - pos;
+    uint32_t len = gfex->length - pos;
     gifputbyte(len, grr); 
     gifputblock(gfex->data + pos, len, grr);
   }
@@ -673,7 +710,7 @@ write_gif(Gif_Stream *gfs, Gif_Writer *grr)
   Gif_Extension *gfex = gfs->extensions;
   Gif_Context gfc;
   
-  gfc.rle_next = Gif_NewArray(Gif_Code, GIF_MAX_CODE);
+  gfc.rle_next = Gif_NewArray(Gif_Code, GIF_MAX_CODE + 1);
   if (!gfc.rle_next)
     goto done;
   
